// Generated by CoffeeScript 1.9.3
(function() {
  var Alarm, EventEmitter, Socket, pad, split,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty,
    slice = [].slice;

  split = require('split');

  EventEmitter = require('events').EventEmitter;

  Socket = require('net').Socket;

  pad = function(num, len) {
    if (len == null) {
      len = 3;
    }
    num = num.toString();
    while (num.length < len) {
      num = "0" + num;
    }
    return num;
  };

  Alarm = (function(superClass) {
    var panelMessageRegex, rfMessageRegex, sendingRegex;

    extend(Alarm, superClass);

    function Alarm(socket1) {
      this.socket = socket1;
      this.setup = bind(this.setup, this);
      this.setup();
    }

    Alarm.prototype.setup = function() {
      return this.socket.pipe(split()).on('data', (function(_this) {
        return function(line) {
          return _this.handleMessage(line.toString('ascii'));
        };
      })(this));
    };


    /*
    Internal: A message has been received and must be handled.
    msg: String message sent by the AD2USB interface.
     */

    panelMessageRegex = /^\[/;

    rfMessageRegex = /^!RFX/;

    sendingRegex = /^!Sending(\.*)done/;

    Alarm.prototype.handleMessage = function(msg) {
      var err;
      try {
        if (msg.match(panelMessageRegex)) {
          return this.handlePanelData(msg);
        } else if (msg.match(rfMessageRegex)) {
          return this.handleRfMessage(msg);
        } else if (msg.match(sendingRegex)) {
          return this.emit('sent');
        }
      } catch (_error) {
        err = _error;
        return this.emit('error', err);
      }
    };


    /*
    Internal: Panel data has been received. Parse it, keep state, and emit events when state changes.
     */

    Alarm.prototype.handlePanelData = function(msg) {
      var armedAway, armedNight, armedStay, beeps, disarmed, lcdtext, parts, sec, sec1, sec2, sections;
      parts = msg.split(',');
      sections = [];
      sec = parts[0].replace(/[\[\]]/g, '');
      sections.push(sec);
      sec1 = sec.split('');
      disarmed = sec1.shift() === '1';
      armedAway = sec1.shift() === '1';
      armedStay = sec1.shift() === '1';
      this.state('backlight', sec1.shift() === '1');
      this.state('programming', sec1.shift() === '1');
      beeps = parseInt(sec1.shift(), 10);
      if (beeps > 0) {
        this.emit('beep', beeps);
      }
      this.state('bypass', sec1.shift() === '1');
      this.state('power', sec1.shift() === '1');
      this.state('chimeMode', sec1.shift() === '1');
      this.state('alarmOccured', sec1.shift() === '1');
      this.state('alarm', sec1.shift() === '1');
      this.state('batteryLow', sec1.shift() === '1');
      this.state('entryDelayOff', sec1.shift() === '1');
      this.state('fireAlarm', sec1.shift() === '1');
      this.state('checkZone', sec1.shift() === '1');
      this.state('perimeterOnly', sec1.shift() === '1');
      sec2 = parts[1];
      sections.push(sec2);
      this.state('fault', sec2);
      sections.push(parts[2].replace(/[\[\]]/g, ''));
      lcdtext = parts[3].replace(/"/g, "");
      if (lcdtext !== this.lcdtext) {
        this.emit('lcdtext', lcdtext);
        this.lcdtext = lcdtext;
      }
      sections.push(parts[3]);
      armedNight = armedStay && lcdtext.indexOf("***NIGHT-STAY***") > -1;
      if (armedNight && armedStay) {
        armedStay = false;
      }
      if (disarmed || armedAway || armedStay || armedNight) {
        if (disarmed && !this.disarmed) {
          this.emit('disarmed');
        } else if (armedAway && !this.armedAway) {
          this.emit('armedAway');
        } else if (armedStay && !this.armedStay) {
          this.emit('armedStay');
        } else if (armedNight && !this.armedNight) {
          this.emit('armedNight');
        }
        this.disarmed = disarmed;
        this.armedAway = armedAway;
        this.armedStay = armedStay;
        this.armedNight = armedNight;
      }
      return this.emit.apply(this, ['raw'].concat(sections));
    };


    /*
    Internal: A RF sensor has reported its status. Parse it, keep state and emit events when state changes.
     */

    Alarm.prototype.handleRfMessage = function(msg) {
      var parts, serial, status;
      parts = msg.replace('!RFX:', '').split(',');
      serial = parts.shift();
      status = pad(parseInt(parts.shift(), 16).toString(2), 8).split('').reverse();
      status = {
        battery: status[1] === '0',
        supervision: status[2] === '0',
        loop1: status[7] === '0',
        loop2: status[5] === '0',
        loop3: status[4] === '0',
        loop4: status[6] === '0'
      };
      this.state("supervision:" + serial, status.supervision);
      this.state("battery:" + serial, status.battery);
      this.state("loop:" + serial + ":1", status.loop1);
      this.state("loop:" + serial + ":2", status.loop2);
      this.state("loop:" + serial + ":3", status.loop3);
      this.state("loop:" + serial + ":4", status.loop4);

      /*
      Raw Emit RF state
       */
      return this.emit("rfraw", serial, status.supervision, status.battery, status.loop1, status.loop2, status.loop3, status.loop4);
    };


    /*
    Internal: Keep track of the state of the named property. If the property changes, then emit
    an event with the new state.
     */

    Alarm.prototype.state = function(name, state) {
      var changed;
      changed = this[name] !== state;
      if (changed) {
        this[name] = state;
        this.emit(name, state);
      }
      return changed;
    };


    /*
    Internal: Send a command to the AD2USB interface.
    
    code: String command to send (i.e. "12341")
    callback: function invoked when interface acknowledges command (optional)
    
    Returns true if command is sent, otherwise false.
     */

    Alarm.prototype.send = function(cmd, callback) {
      this.once('sent', function(msg) {
        if (callback) {
          return callback(null, msg);
        }
      });
      return this.socket.write(cmd);
    };


    /*
    Public: Check armed status
    
    Returns true if alarm is armed in stay or away mode, otherwise false
     */

    Alarm.prototype.isArmed = function() {
      return this.armedStay || this.armedAway;
    };


    /*
    Public: Arm the alarm in away mode.
    
    code: The user code to use to arm the alarm.
    callback: function invoked when interface acknowledegs command (optional)
    
    Returns true if command is sent, otherwise false
     */

    Alarm.prototype.armAway = function(code, callback) {
      if (code) {
        return this.send(code + "2", callback);
      }
    };


    /*
    Public: Arm the alarm in away stay mode.
    
    code: The user code to use to arm the alarm.
    callback: function invoked when interface acknowledegs command (optional)
    
    Returns true if command is sent, otherwise false
     */

    Alarm.prototype.armStay = function(code, callback) {
      if (code) {
        return this.send(code + "3", callback);
      }
    };


    /*
    Public: Arm the alarm in night stay mode.
    
    code: The user code to use to arm the alarm.
    callback: function invoked when interface acknowledegs command (optional)
    
    Returns true if command is sent, otherwise false
     */

    Alarm.prototype.armNight = function(code, callback) {
      if (code) {
        return this.send(code + "33", callback);
      }
    };


    /*
    Public: Disarm the alarm.
    
    code: The user code to use to disarm the alarm.
    callback: function invoked when interface acknowledegs command (optional)
    
    Returns true if command is sent, otherwise false
     */

    Alarm.prototype.disarm = function(code, callback) {
      if (code) {
        return this.send(code + "1", callback);
      }
    };


    /*
    Public: Bypass a zone.
    
    code: The user code to use to bypass
    zone: The zone number to bypass
    callback: function invoked when interface acknowledegs command (optional)
    
    Returns true if command is sent, otherwise false
     */

    Alarm.prototype.bypass = function(code, zone, callback) {
      if (code) {
        return this.send(code + "6" + zone, callback);
      }
    };


    /*
    Public: Connect to the AD2USB device using a TCP socket.
    
    ip: String IP address of interface
    port: Integer TCP port of interface (optional, defaults to 4999)
    callback: invoked once the connection has been established (optional)
     */

    Alarm.connect = function() {
      var alarm, args, callback, ip, port, ref, socket;
      args = 1 <= arguments.length ? slice.call(arguments, 0) : [];
      if (typeof args[args.length - 1] === 'function') {
        callback = args.pop();
      }
      ip = args.shift();
      port = (ref = args.shift()) != null ? ref : 4999;
      socket = new Socket({
        type: 'tcp4'
      });
      alarm = new Alarm(socket);
      socket.connect(port, ip, callback);
      return alarm;
    };

    return Alarm;

  })(EventEmitter);

  module.exports = Alarm;

}).call(this);
