# template [![NPM version](https://badge.fury.io/js/template.png)](http://badge.fury.io/js/template)

> An easy-to-use [Lo-Dash template](http://lodash.com/docs#template) processing library.

## Quickstart

Install with [npm](npmjs.org):

```bash
npm i template --save
```

## Methods

### template

By default the `template()` method expects the following parameters:

```js
template(text, data, [options])
```

For example, the following:

```js
var template = require('template');

template('Name: <%= name %>', {name: 'Jon'});
```
results in:

```
Name: Jon
```

### template.read

To read a file from the file system before processing, use `template.read`:

```js
template.read('file.md', data, options);
```

### template.copy

A convenience method for synchronously copying files from A to B.

```js
template.copy(src, dest, data, options);
```

Pass `{process: false}` to the options disable template processing.


## Options
Template supports all [Lo-Dash template](http://lodash.com/docs#template) options, as well as a few extras.

### custom delimiters
All of the options from the [delims](https://github.com/jonschlinkert/delims) library may be passed to the options object.

For example, this:

```js
template('Hi, my name is {%= name %}', {name: 'Jon Schlinkert'}, {delims: ['{%', '%}']});
```

Results in:

```
Hi, my name is Jon Schlinkert
```

## Valid templates
Any of the following expressions may be used in templates:

```js
// Strings
"foo"

// Property strings
foo
foo.bar
foo.bar.baz

// Method calls, passed as properties on the data object
one()
two.three()

// Lo-Dash Mixins
_.foo()
_.foo(bar)
_.foo("baz")
```

## 3rd Party Libs

**[underscore.string][str]**

All of the methods from [underscore.string][str] are available on the `_.str` object. Additionally, all of the _non-conflicting_ methods are available on the `_` Lo-Dash namespace. See the [underscore.string documentation][str-docs] for more details.


## Authors
**Jon Schlinkert**

+ [github/jonschlinkert](https://github.com/jonschlinkert)
+ [twitter/jonschlinkert](http://twitter.com/jonschlinkert)

**Brian Woodward**

+ [github/doowb](https://github.com/doowb)
+ [twitter/doowb](http://twitter.com/jonschlinkert)


## License
Copyright (c) 2014 [Jon Schlinkert](http://twitter.com/jonschlinkert), [Brian Woodward](http://twitter.com/doowb), contributors.
Released under the [MIT license](./LICENSE-MIT)

[str]: https://github.com/epeli/underscore.string
[str-docs]: https://github.com/epeli/underscore.string#nodejs-installation